<?php

namespace App\Listeners\Frontend\Auth;

use App\Events\Frontend\Auth\UserConfirmed;
use App\Events\Frontend\Auth\UserLoggedIn;
use App\Events\Frontend\Auth\UserLoggedOut;
use App\Events\Frontend\Auth\UserProviderRegistered;
use App\Events\Frontend\Auth\UserRegistered;
use App\Models\Boards\Board;
use App\Models\Lab;
use App\Models\Messages\ExtendedMessage;
use App\Models\Messages\ExtendedThread;
use Carbon\Carbon;
use Cmgmyr\Messenger\Models\Participant;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Storage;

/**
 * Class UserEventListener.
 */
class UserEventListener
{
    /**
     * @param $event
     */
    public function onLoggedIn($event)
    {
        $ip_address = request()->getClientIp();

        // Update the logging in users time & IP
        $event->user->fill([
            'last_login_at' => now()->toDateTimeString(),
            'last_login_ip' => $ip_address,
        ]);

        // Update the timezone via IP address
        $geoip = geoip($ip_address);

        if ($event->user->timezone !== $geoip['timezone']) {
            // Update the users timezone
            $event->user->fill([
                'timezone' => $geoip['timezone'],
            ]);
        }

        $event->user->save();

        // add bonus boards from templates here
        $templateboards = Board::bonusTemplates()->get();
        $already_done = Board::forUser($event->user->id)->whereNotNull('from_template')->pluck('from_template');
        foreach ($templateboards as $templateboard) {
            if (!$already_done->contains($templateboard->id)){
                $duplicateBoard = $templateboard->replicate();
                $data_filename = uniqid("board_{$duplicateBoard->id}", true);
                Storage::disk('boards')->copy($templateboard->data_location, $data_filename);
                $duplicateBoard->data_location = $data_filename;
                $duplicateBoard->is_template = null;
                $duplicateBoard->lab_id = null;
                $duplicateBoard->board_folder_id = null;
                $duplicateBoard->bonus_template = null;
                $duplicateBoard->from_template = $templateboard->id;
                $duplicateBoard->exclude_from_limit = 'true';
                $duplicateBoard->user_id = $event->user->id;
                $duplicateBoard->save();
            }
        }

        logger('User Logged In: ' . $event->user->full_name);
    }

    /**
     * @param $event
     */
    public function onLoggedOut($event)
    {
        logger('User Logged Out: ' . $event->user->full_name);
    }

    /**
     * @param $event
     */
    public function onRegistered($event)
    {
        //@TODO make this more elegant

        // send a welcome notification to the user
        $message = "Welcome to Graphics et al. We hope you like it here. Please don't forget to confirm your email(It might have gone to spam).<br/>We love you.";
        $notification = new ExtendedThread();
        $notification->makeNotification('Welcome!', $message, [], [$event->user->id], 0, 'Graphics et al');

        // add universal template boards here
        $templateboards = Board::baseTemplates()->get();

        foreach ($templateboards as $templateboard) {
            $duplicateBoard = $templateboard->replicate();
            $data_filename = uniqid("board_{$duplicateBoard->id}", true);
            Storage::disk('boards')->copy($templateboard->data_location, $data_filename);
            $duplicateBoard->data_location = $data_filename;
            $duplicateBoard->is_template = null;
            $duplicateBoard->base_template = null;
            $duplicateBoard->lab_id = null;
            $duplicateBoard->board_folder_id = null;
            $duplicateBoard->exclude_from_limit = 'true';
            $duplicateBoard->user_id = $event->user->id;
            $duplicateBoard->save();
        }
        // temporarily add new users to beta testing lab
        $event->user->assignRole('lab member');
        $betalab = Lab::find(1);

        if (!$betalab->members->contains($event->user)) {
            $betalab->members()->attach($event->user);
        }
    }

    /**
     * @param $event
     */
    public function onProviderRegistered($event)
    {
        logger('User Provider Registered: ' . $event->user->full_name);
        // send a welcome notification to the user
        $message = "Welcome to Graphics et al. We hope you like it here.<br/>We love you.";
        $notification = new ExtendedThread();
        $notification->makeNotification('Welcome!', $message, [], [$event->user->id], 0, 'Graphics et al');
    }

    /**
     * @param $event
     */
    public function onConfirmed($event)
    {
        logger('User Confirmed: ' . $event->user->full_name);
    }

    /**
     * Register the listeners for the subscriber.
     *
     * @param \Illuminate\Events\Dispatcher $events
     */
    public function subscribe($events)
    {
        $events->listen(
            UserLoggedIn::class,
            'App\Listeners\Frontend\Auth\UserEventListener@onLoggedIn'
        );

        $events->listen(
            UserLoggedOut::class,
            'App\Listeners\Frontend\Auth\UserEventListener@onLoggedOut'
        );

        $events->listen(
            UserRegistered::class,
            'App\Listeners\Frontend\Auth\UserEventListener@onRegistered'
        );

        $events->listen(
            UserProviderRegistered::class,
            'App\Listeners\Frontend\Auth\UserEventListener@onProviderRegistered'
        );

        $events->listen(
            UserConfirmed::class,
            'App\Listeners\Frontend\Auth\UserEventListener@onConfirmed'
        );
    }
}
